#-*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2014 Therp BV        (<http://www.therp.nl>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

'''All functionality to enable fixed prices in pricelists.'''

from openerp.osv import orm, fields
from openerp.tools.translate import _


class ProductPricelistItem(orm.Model):
    '''Inherit existing model to add functionality for fixed prices'''
    _inherit = 'product.pricelist.item'

    def _modify_vals(self, cr, uid, vals, browse_obj=None, context=None):
        '''Ensure consistent values for fixed pricelists.
        The passed vals parameter is used for both input and output.'''
        # First determine wether a fixed price should apply
        fixed_price = False
        if browse_obj:
            fixed_price = browse_obj.fixed_price
        else:
            # new record
            fixed_price = (
                ('fixed_price' in vals and vals['fixed_price']) or
                ('default_fixed_price' in vals
                    and vals['default_fixed_price']) or
                False
            )
        if not fixed_price:
            return
        # If we get here, we have a fixed price, because we need some
        # reference to a price in product (even though its influence will be
        # eliminated by our price_discount), we use the standard price
        model_model = self.pool.get('ir.model.data')
        standard_price_type = model_model.get_object_reference(
            cr, uid, 'product', 'standard_price')[1]
        vals.update({
            'price_discount': -1.0,
            'price_round': 0.0,
            'base': standard_price_type,
            'price_min_margin': 0.0,
            'price_max_margin': 0.0,
        })

    def create(self, cr, uid, vals, context=None):
        '''Make sure fixed pricelist items have consistent values'''
        self._modify_vals(cr, uid, vals, browse_obj=None, context=context)
        return super(ProductPricelistItem, self).create(
            cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        '''Override write method. We have to take into account that
        we only get one set of values, but that the change might effect
        multiple ids. But because the computations we do might be different
        depending on the current values in the database, we will setup a loop
        and compute and update each record separately.
        '''
        for item_id in ids:
            browse_records = self.browse(cr, uid, [item_id], context=context)
            browse_obj = browse_records[0]
            self._modify_vals(
                cr, uid, vals, browse_obj=browse_obj, context=context)
            super(ProductPricelistItem, self).write(
                cr, uid, [item_id], vals, context=context)
        return True

    def onchange_fixed_price(
            self, cr, uid, ids, fixed_price, context=None):
        res = {}
        if fixed_price:
            vals = {'fixed_price': True}
            self._modify_vals(cr, uid, vals, context=context)
            res['value'] = vals
        return res

    _columns = {
        'fixed_price': fields.boolean('Fixed price'),
    }

    def _check_fixed_price(self, cr, uid, ids):
        '''Ensure fixed prices always refer to a specific product.'''
        for this_obj in self.browse(cr, uid, ids):
            if not this_obj.fixed_price:
                return True
            if not this_obj.product_id:
                raise orm.except_orm(
                    _('Validation error!'),
                    _('Product required for fixed price item.')
                )
            # Values for price_discount and price_round will not be checked,
            # because create and write will automagically set appropiate
            # values.
        return True

    _constraints = [
        (_check_fixed_price,
            'invalid values for fixed price', ['fixed_price']),
    ]

